"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useNewControllableValue = void 0;

var react_1 = require("react");

var ahooks_1 = require("ahooks");

function useNewControllableValue(options) {
  var value = options.value,
      defaultValue = options.defaultValue,
      onChange = options.onChange;
  var update = ahooks_1.useUpdate();
  var stateRef = react_1.useRef(value !== undefined ? value : defaultValue);

  if (value !== undefined) {
    stateRef.current = value;
  }

  var setState = react_1.useCallback(function (v) {
    if (value === undefined) {
      stateRef.current = v;
      update();
    }

    onChange === null || onChange === void 0 ? void 0 : onChange(v);
  }, [value, update, onChange]);
  return [stateRef.current, setState];
}

exports.useNewControllableValue = useNewControllableValue;