"use strict";

var __createBinding = this && this.__createBinding || (Object.create ? function (o, m, k, k2) {
  if (k2 === undefined) k2 = k;
  Object.defineProperty(o, k2, {
    enumerable: true,
    get: function get() {
      return m[k];
    }
  });
} : function (o, m, k, k2) {
  if (k2 === undefined) k2 = k;
  o[k2] = m[k];
});

var __setModuleDefault = this && this.__setModuleDefault || (Object.create ? function (o, v) {
  Object.defineProperty(o, "default", {
    enumerable: true,
    value: v
  });
} : function (o, v) {
  o["default"] = v;
});

var __importStar = this && this.__importStar || function (mod) {
  if (mod && mod.__esModule) return mod;
  var result = {};
  if (mod != null) for (var k in mod) {
    if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
  }

  __setModuleDefault(result, mod);

  return result;
};

var __importDefault = this && this.__importDefault || function (mod) {
  return mod && mod.__esModule ? mod : {
    "default": mod
  };
};

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Swiper = void 0;

var react_1 = __importStar(require("react"));

var native_props_1 = require("../../utils/native-props");

var with_default_props_1 = require("../../utils/with-default-props");

var classnames_1 = __importDefault(require("classnames"));

var swiper_item_1 = require("./swiper-item");

var dev_log_1 = require("../../utils/dev-log");

var web_1 = require("@react-spring/web");

var react_use_gesture_1 = require("react-use-gesture");

var rubberband_1 = require("../../utils/rubberband");

var page_indicator_1 = __importDefault(require("../page-indicator"));

var staged_components_1 = require("staged-components");

var use_ref_state_1 = require("../../utils/use-ref-state");

var defaultProps = {
  defaultIndex: 0,
  allowTouchMove: true,
  autoplay: false,
  autoplayInterval: 3000,
  loop: true
};
exports.Swiper = react_1.forwardRef(staged_components_1.staged(function (p, ref) {
  var props = with_default_props_1.mergeProps(defaultProps, p);

  var _react_1$useMemo = react_1.useMemo(function () {
    var count = 0;
    var validChildren = react_1["default"].Children.map(props.children, function (child) {
      if (!react_1["default"].isValidElement(child)) return null;

      if (child.type !== swiper_item_1.SwiperItem) {
        dev_log_1.devWarning('Swiper', 'The children of `Swiper` must be `Swiper.Item` components.');
        return null;
      }

      count++;
      return child;
    });
    return {
      validChildren: validChildren,
      count: count
    };
  }, [props.children]),
      validChildren = _react_1$useMemo.validChildren,
      count = _react_1$useMemo.count;

  if (count === 0) {
    dev_log_1.devWarning('Swiper', '`Swiper` needs at least one child.');
    return null;
  }

  return function () {
    var loop = props.loop;

    if (count === 1 && loop) {
      dev_log_1.devWarning('Swiper', '`Swiper` needs at least two children to enable loop.');
      loop = false;
    }

    var trackRef = react_1.useRef(null);

    function getWidth() {
      var track = trackRef.current;
      if (!track) return 0;
      return track.offsetWidth;
    }

    var _react_1$useState = react_1.useState(props.defaultIndex),
        current = _react_1$useState[0],
        setCurrent = _react_1$useState[1];

    var _use_ref_state_1$useR = use_ref_state_1.useRefState(false),
        dragging = _use_ref_state_1$useR[0],
        setDragging = _use_ref_state_1$useR[1],
        draggingRef = _use_ref_state_1$useR[2];

    var _web_1$useSpring = web_1.useSpring(function () {
      return {
        x: rubberband_1.bound(current, 0, count - 1) * 100,
        config: {
          tension: 200,
          friction: 30
        },
        onRest: function onRest() {
          if (draggingRef.current) return;
          var rawX = x.get();
          var totalWidth = 100 * count;
          var standardX = modulus(rawX, totalWidth);
          if (standardX === rawX) return;
          api.start({
            x: standardX,
            immediate: true
          });
        }
      };
    }, [count]),
        x = _web_1$useSpring[0].x,
        api = _web_1$useSpring[1];

    var bind = react_use_gesture_1.useDrag(function (state) {
      var width = getWidth();
      if (!width) return;
      var _state$movement = state.movement,
          mx = _state$movement[0];

      if (state.last) {
        window.setTimeout(function () {
          setDragging(false);
        });
        var index = Math.round((mx + state.vxvy[0] * 100) / width);
        swipeTo(index);
      } else {
        setDragging(true);
        api.start({
          x: mx * 100 / width,
          immediate: true
        });
      }
    }, {
      transform: function transform(_ref) {
        var x = _ref[0],
            y = _ref[1];
        return [-x, y];
      },
      initial: function initial() {
        var width = getWidth();
        return [x.get() / 100 * width, 0];
      },
      bounds: function bounds() {
        if (loop) return {};
        var width = getWidth();
        return {
          left: 0,
          right: (count - 1) * width
        };
      },
      rubberband: true,
      axis: 'x',
      experimental_preventWindowScrollY: true
    });

    function swipeTo(index) {
      var _a, _b;

      if (loop) {
        var i = modulus(index, count);
        setCurrent(i);
        (_a = props.onIndexChange) === null || _a === void 0 ? void 0 : _a.call(props, i);
        api.start({
          x: index * 100
        });
      } else {
        var _i = rubberband_1.bound(index, 0, count - 1);

        setCurrent(_i);
        (_b = props.onIndexChange) === null || _b === void 0 ? void 0 : _b.call(props, _i);
        api.start({
          x: _i * 100
        });
      }
    }

    function swipeNext() {
      swipeTo(Math.round(x.get() / 100) + 1);
    }

    function swipePrev() {
      swipeTo(Math.round(x.get() / 100) - 1);
    }

    react_1.useImperativeHandle(ref, function () {
      return {
        swipeTo: swipeTo,
        swipeNext: swipeNext,
        swipePrev: swipePrev
      };
    });
    var autoplay = props.autoplay,
        autoplayInterval = props.autoplayInterval;
    react_1.useEffect(function () {
      if (!autoplay || dragging) return;
      var interval = window.setInterval(function () {
        swipeNext();
      }, autoplayInterval);
      return function () {
        window.clearInterval(interval);
      };
    }, [autoplay, autoplayInterval, dragging]);
    return native_props_1.withNativeProps(props, react_1["default"].createElement("div", {
      className: 'adm-swiper'
    }, react_1["default"].createElement("div", Object.assign({
      className: classnames_1["default"]('adm-swiper-track', {
        'adm-swiper-track-allow-touch-move': props.allowTouchMove
      }),
      onClickCapture: function onClickCapture(e) {
        if (draggingRef.current) {
          e.stopPropagation();
          e.preventDefault();
        }
      }
    }, props.allowTouchMove ? bind() : {}), react_1["default"].createElement("div", {
      className: 'adm-swiper-track-inner',
      ref: trackRef
    }, react_1["default"].Children.map(validChildren, function (child, index) {
      return react_1["default"].createElement(web_1.animated.div, {
        className: 'adm-swiper-slide',
        style: {
          x: x.to(function (x) {
            var position = -x + index * 100;

            if (loop) {
              var totalWidth = count * 100;
              position = modulus(position + 100, totalWidth) - 100;
            }

            return position + "%";
          }),
          left: "-" + index * 100 + "%"
        }
      }, child);
    }))), props.indicator === undefined ? react_1["default"].createElement("div", {
      className: 'adm-swiper-indicator'
    }, react_1["default"].createElement(page_indicator_1["default"], Object.assign({}, props.indicatorProps, {
      total: count,
      current: current
    }))) : props.indicator(count, current)));
  };
}));

function modulus(value, division) {
  var remainder = value % division;
  return remainder < 0 ? remainder + division : remainder;
}